from ali_autodrive.parallel_compute.biz_enum.TaskStatusEnum import TaskStatusEnum

'''
任务管理实现类，控制任务执行
'''
# 常量
STATUS = "status"
HEALTH = "health"
NEW_CONTEXT = "new_context"
CHECK_POINT = "check_point"
DATA_PARTITION_ID = "dataPartitionId"


class ExecutorAgentManager:
    def __init__(self, parallel_compute_manager):
        self.parallel_compute_manager = parallel_compute_manager

    # 健康检查，每个10s调用一次
    def check_health(self, context):
        return self.__get_task(context).health

    # 一个task执行完成，调用一次
    def get_context(self, context):
        return self.__get_task(context).new_context

    # 每个1min调用一次
    def get_check_point(self, context):
        return self.__get_task(context).check_point

    # 一个task要执行，调用一次
    def run(self, context):
        return self.parallel_compute_manager.submit_task(context)

    # task执行被上层kill，调用一次
    def kill(self, context):
        return self.parallel_compute_manager.kill(context)

    # task执行中会被调用
    def status(self, context):
        return self.__get_task(context).status

    def __get_task(self, context):
        task = self.parallel_compute_manager.get_task(context)
        if task is None:
            raise RuntimeError('Task not exists, context:', context)
        return task
