from ali_autodrive.parallel_compute.utils.http_util import *
from ali_autodrive.parallel_compute.executor_agent.LogUtil import *

import json

'''
数据分片读客户端
'''

# 常量
# 上下文
CONTEXT_KEY = "context"
# 数据类型
SOURCE_TYPE = "sourceType"
# 健康状态为健康
HEALTH_TRUE = "true"
# 返回结果，成功
RESPONSE_SUCCESS = "success"
# http返回结果数据
RESPONSE_DATA = "data"
# 数据分片ID
DATA_PARTITION_ID = "dataPartitionId"
# 数据分片ID列表
DATA_PARTITION_ID_LIST = "dataPartitionIds"

# 文件根路径
ROOT_PATH = "rootPath"
# 文件数，根节点
ROOT = "root"
# 分页，当前页数
PAGE_NUM = "pageNum"
# 分页，页面大小
PAGE_SIZE = "pageSize"
# 分页，总记录数
TOTAL_SIZE = "totalSize"
# 框架工作空间
WORK_SPACE = "workspace"
# 节点配置参数
NODE_PARAMETERS = "parameters"
# 节点配置参数 OSS资源
OSS_SOURCE_CODE = "ossSourceCode"
# 节点配置参数 OSS资源
OSS_SOURCE_CODE_KEY = "cloudSourceCode"
# 扫描方式
SCAN_TYPE = "scanType"
# 节点配置参数 OSS路径
OSS_PREFIX = "ossPrefix"


class ReadClient:
    def __init__(self, processor_task):
        self.processor_task = processor_task
        # 初始化log服务
        self.logger = default_logger

    # 读取数据分片信息
    def get_data_partition(self):
        # 更新健康状态
        self.processor_task.modify_health(HEALTH_TRUE)

        # 获取当前数据分片
        data_partition_id = self.processor_task.data_partition_id

        # 获取分片描述
        data_dictionary = {DATA_PARTITION_ID: data_partition_id}
        data = get_data_partition(json.dumps(data_dictionary))
        self.logger.info("Get data partition:", data)

        # 返回结果
        return data

    # 根据扫描文件列表
    def scan_file_tree_by_partition_ids(self, data_partition_ids):
        # 更新健康状态
        self.processor_task.modify_health(HEALTH_TRUE)

        # 请求参数组装
        data_dictionary = {WORK_SPACE: self.processor_task.context[WORK_SPACE],
                           SOURCE_TYPE: "OSS",
                           DATA_PARTITION_ID_LIST: data_partition_ids}

        # 读取文件树
        data = scan_file_tree(json.dumps(data_dictionary))
        self.logger.info("Scan file tree by root path", data)

        # 返回文件树，JSON格式
        return data[ROOT]

    # 加载非结构数据
    # 将数据加载到本地文件路径，本期支持OSS
    def load_unstructured_data_partition_by_partition_ids(self, data_partition_ids):
        # 更新健康状态
        self.processor_task.modify_health(HEALTH_TRUE)

        # 请求参数组装
        data_dictionary = {WORK_SPACE: self.processor_task.context[WORK_SPACE],
                           SOURCE_TYPE: "OSS",
                           DATA_PARTITION_ID_LIST: data_partition_ids}

        data = load_unstructured_data_partition(json.dumps(data_dictionary))
        self.logger.info("Load unstructured data partition", data)

        return True

    def get_partition_local_desc_file(self):
        # 更新健康状态
        self.processor_task.modify_health(HEALTH_TRUE)

        # 获取当前数据分片
        data_partition_id = self.processor_task.data_partition_id

        # 获取分片描述
        data_dictionary = {WORK_SPACE: self.processor_task.context[WORK_SPACE], DATA_PARTITION_ID: data_partition_id}
        data = get_partition_local_desc_file(json.dumps(data_dictionary))
        self.logger.info("Get data partition local desc file:", data)

        return data["localDescFile"]
