from ali_autodrive.parallel_compute.utils.http_util import *
from ali_autodrive.parallel_compute.executor_agent.LogUtil import *

import json

'''
数据分片写客户端
'''

# 常量
# 数据类型
DATA_TYPE_KEY = "dataType"
# 存储实际内容
CONTENT_LIST = "contents"
# 存储实际内容
FINISHED = "finished"
# 健康状态为健康
HEALTH_TRUE = "true"
# 处理结果，成功
RESULT_TRUE = "true"
# http请求返回成功
RESPONSE_SUCCESS = "success"
# http请求返回数据
RESPONSE_DATA = "data"
# 数据分片
DATA_PARTITION_ID = "dataPartitionId"
# 工作空间
WORK_SPACE = "workspace"
# 实例CODE
INSTANCE_CODE = "instanceCode"
# 节点动作
STEP_TYPE = "stepType"
# 拆分节点拆分分片索引
PARTITION_INDEX = "partitionIndex"
# 节点配置参数
NODE_PARAMETERS = "parameters"
# 节点配置参数 OSS资源
OSS_SOURCE_CODE = "ossSourceCode"
ES_SOURCE_CODE = "esSourceCode"
# 节点配置参数 OSS路径
OSS_PREFIX = "ossPrefix"
# 节点配置参数 ES索引
ES_INDEX = "esIndex"
# 父节点上下文列表
PARENT_CONTEXT_LIST = "parentContext"
# 数据分片内容
CONTENT_MAP = "contentMap"
SOURCE_CODE = "sourceCode"
PREFIX = "prefix"
INDEX_NAME = "indexName"
# 任务流实例
FLOW_INSTANCE_ID = "flowInstanceId"
FLOW_CODE = "flowCode"
FLOW_NAME = "flowName"
NODE_CODE = "nodeCode"
NODE_NAME = "nodeName"
# 逻辑拆分，指定父分片根路径
PARENT_PARTITION_ROOT = "rootPath"
LOCAL_PATH = "localPath"
OSS_PATH = "ossPath"

class WriteClient:
    def __init__(self, processor_task):
        self.processor_task = processor_task
        # 初始化log服务
        self.logger = default_logger

    # OSS:content为本地文件列表, OSS_LOGIC:content为OSS文件列表
    # dataType数据类型  OSS/OSS_LOGIC
    # dataPartitionId 标记存储的数据分片
    # context节点上下文
    def save_data_partition(self, contents):
        # 更新健康状态
        self.processor_task.modify_health(HEALTH_TRUE)

        # 获取存储分片
        data_partition_id = self.__get_data_partition_id(self.processor_task.context)

        # 构造请求数据
        if self.processor_task.has_load_parent_data:
            data_type = "OSS"
        else:
            data_type = "OSS_LOGIC"
        data_dictionary = {WORK_SPACE: self.processor_task.context[WORK_SPACE], DATA_PARTITION_ID: data_partition_id,
                           DATA_TYPE_KEY: data_type, CONTENT_LIST: contents}

        # 存储分片数据
        save_data_partition(json.dumps(data_dictionary))

        # 如果是拆分节点，及时提交
        if self.processor_task.context[STEP_TYPE] == "PARTITION":
            self.submit_data_partition(data_partition_id)
            self.processor_task.data_partition_index += 1
            if self.processor_task.data_partition_list is None:
                self.processor_task.data_partition_list = [data_partition_id]
            else:
                self.processor_task.data_partition_list.append(data_partition_id)

        return True

    def save_structured_data(self, index, contents):
        # 获取存储分片
        data_partition_id = self.__get_data_partition_id(self.processor_task.context)

        data_dictionary = {DATA_PARTITION_ID: data_partition_id,
                           DATA_TYPE_KEY: "ES", "index": index, CONTENT_LIST: contents}

        # 存储分片数据
        save_data_partition(json.dumps(data_dictionary))

        return True

    def save_unstructured_data(self, local_path, oss_path):
        # 获取存储分片
        data_partition_id = self.__get_data_partition_id(self.processor_task.context)

        data_dictionary = {DATA_PARTITION_ID: data_partition_id,
                           DATA_TYPE_KEY: "OSS_PCD_COMPRESSION", LOCAL_PATH: local_path, OSS_PATH: oss_path}

        # 存储分片数据
        save_data_partition(json.dumps(data_dictionary))

        return True

    def submit_data_partition(self, data_partition_id):
        # 更新健康状态
        self.processor_task.modify_health(HEALTH_TRUE)

        # 构造请求数据
        data_dictionary = {WORK_SPACE: self.processor_task.context[WORK_SPACE], DATA_PARTITION_ID: data_partition_id}

        # 提交分片数据
        submit_data_partition(json.dumps(data_dictionary))
        return True

    def __get_data_partition_id(self, context):
        if self.processor_task.data_partition_id is not None:
            return self.processor_task.data_partition_id

        # 构造请求数据
        data_dictionary = {INSTANCE_CODE: context[INSTANCE_CODE], STEP_TYPE: context[STEP_TYPE],
                           PARTITION_INDEX: self.processor_task.data_partition_index}

        # 查询分片
        data_partition = get_data_partition(json.dumps(data_dictionary))
        data_partition_id = None
        if data_partition is None:
            data_partition_id = self.create_data_partition(context, False)
        else:
            data_partition_id = data_partition[DATA_PARTITION_ID]

        return data_partition_id

    def create_data_partition(self, context, init):
        step_type = context[STEP_TYPE]
        root_path = None
        if init:
            step_type = "INIT"
        elif self.processor_task.has_load_parent_data is False:
            parent_data_dictionary = {
                DATA_PARTITION_ID: self.processor_task.context[PARENT_CONTEXT_LIST][0][DATA_PARTITION_ID]}
            parent_data = get_data_partition(json.dumps(parent_data_dictionary))
            root_path = parent_data[CONTENT_MAP]["OSS"][PARENT_PARTITION_ROOT]

        node_parameters = context[NODE_PARAMETERS]
        content_oss = {SOURCE_CODE: node_parameters[OSS_SOURCE_CODE], PREFIX: node_parameters[OSS_PREFIX],
                       PARENT_PARTITION_ROOT: root_path}
        content_es = {SOURCE_CODE: node_parameters[ES_SOURCE_CODE]}
        content_map = {"ES": content_es, "OSS": content_oss}

        # 构造创建数据
        data_dictionary = {INSTANCE_CODE: context[INSTANCE_CODE],
                           STEP_TYPE: step_type,
                           FLOW_CODE: context[FLOW_CODE],
                           FLOW_NAME: context[FLOW_NAME],
                           NODE_CODE: context[NODE_CODE],
                           NODE_NAME: context[NODE_NAME],
                           FLOW_INSTANCE_ID: context[FLOW_INSTANCE_ID],
                           PARTITION_INDEX: self.processor_task.data_partition_index,
                           CONTENT_MAP: content_map}

        data = create_data_partition(json.dumps(data_dictionary))
        return data[DATA_PARTITION_ID]
