# -*-coding:utf-8-*-

from flask import Flask, request
from .LogUtil import get_log, default_logger, filename, default_file
import sys
import os
from . import ParamKey

_app = Flask(__name__)

_executor_agent_manager = None


def register(agent_manager):
    global _executor_agent_manager
    _executor_agent_manager = agent_manager


def start():
    if _executor_agent_manager == None:
        default_logger.error("not set executor agent manager, exit 1")
        sys.exit(1)

    # TODO: get env, set port
    _app.run()


@_app.get("/checkHealth")
def _check_health():
    return 'ok'


@_app.get("/task/checkHealth")
def _check_task_health():
    instance_code = request.args.get(ParamKey.INSTANCE_CODE)
    return _executor_agent_manager.check_health({ParamKey.INSTANCE_CODE: instance_code})


@_app.post("/context/save")
def _get_context():
    body_param = request.get_json()
    return _executor_agent_manager.get_context(body_param)


@_app.post("/checkpoint/save")
def _get_checkpoint():
    body_param = request.get_json()
    return _executor_agent_manager.get_check_point(body_param)


@_app.post("/instance/run")
def _instance_run():
    body_param = request.get_json()
    # 返回true， false， 字符串返回
    return _executor_agent_manager.run(body_param)


@_app.post("/instance/kill")
def _instance_kill():
    body_param = request.get_json()
    # 返回true、false， 字符串返回
    return _executor_agent_manager.kill(body_param)


@_app.get("/instance/status")
def _instance_status():
    instance_code = request.args.get(ParamKey.INSTANCE_CODE)
    # 返回：running 字符串返回
    return _executor_agent_manager.status({ParamKey.INSTANCE_CODE: instance_code})


@_app.get("/instance/log")
def _instance_log():
    instance_code = request.args.get(ParamKey.INSTANCE_CODE)
    log_start_len = int(request.args.get(ParamKey.LOG_START_LEN, 0))
    log_part_len = int(request.args.get(ParamKey.LONG_PART_LEN, 1000))
    log_file_name = filename(instance_code)
    return _read_log(log_file_name, log_start_len, log_part_len)


@_app.get("/default/log")
def _default_log():
    log_start_len = int(request.args.get(ParamKey.LOG_START_LEN, 0))
    log_part_len = int(request.args.get(ParamKey.LONG_PART_LEN, 1000))
    log_file_name = filename(default_file)
    return _read_log(log_file_name, log_start_len, log_part_len)


def _read_log(log_file_name, log_start_len, log_part_len):
    if not os.path.exists(log_file_name):
        raise RuntimeError('%s logfile not exists' % log_file_name)
    res = ''
    with open(log_file_name, 'r', encoding='utf-8') as fp:
        line = fp.readline()
        counts = 0
        while line and counts < log_start_len + log_part_len:
            if counts >= log_start_len:
                res += line
            line = fp.readline()
            counts += 1

    return res


if __name__ == "__main__":
    start()
