import datetime as dt
import time
from threading import Timer
from ali_autodrive.parallel_compute.utils.http_util import *
import configparser

config = configparser.ConfigParser()
config.read("./config.ini", encoding="utf-8")
debug_mode = config.getboolean("debug", "debug_mode")
workspace = config.get("init", "workspace")
oss_prefix = config.get("init", "ossPrefix")
transform_params = config.get("init", "transformParams")

CONNECT_ADDRESS = "http://127.0.0.1:5000"
INSTANCE_RUN = "/instance/run"
GET_STATUS = "/instance/status"
CONTEXT = "/context/save"
DATA_PARTITION_ID_LIST = "dataPartitionIds"

partition_data = {
    "instanceCode": "instanceCode_test_2022090401",
    "flowCode": "flowCode",
    "flowName": "并行计算测试",
    "nodeCode": "nodeCode",
    "nodeName": "标准化节点",
    "stepType": "PARTITION",
    "workspace": "/Users/icyore/new_workspace",
    "parameters": {
        "ossSourceCode": "b0d3d07d5a974a8488722c85c8b545d1",
        "ossPrefix": "parallel_compute/rosbag", "esSourceCode": "4aaa18da3f494af1b14ad5352c1ab598"}
}

parallel_data = {
    "instanceCode": "instanceCode_test_2022083101",
    "flowCode": "taskflow_33c0a0d7d41f4e3d87e2af87387ae4c4",
    "flowName": "并行计算测试",
    "nodeCode": "nodeCode",
    "nodeName": "标准化节点",
    "stepType": "PARALLEL",
    "workspace": "/Users/icyore/new_workspace",
    "parameters": {
        "ossSourceCode": "b0d3d07d5a974a8488722c85c8b545d1",
        "esSourceCode": "4aaa18da3f494af1b14ad5352c1ab598",
        "ossPrefix": "parallel_compute/rosbag",
        "transformParams": "{\"vehicleId\":\"并行计算测试车\",\"topic_list\":[{\"sensor_type\":\"lidar\",\"sensor_type_id\":\"lidar001\",\"topic_type\":\"sensor_msgs/PointCloud2\",\"topic_path\":\"/livox_lidar_front/compensator/simple_PointCloud2\"},{\"sensor_type\":\"lidar\",\"sensor_type_id\":\"lidar002\",\"topic_type\":\"sensor_msgs/PointCloud2\",\"topic_path\":\"/rslidar_fuse_32/simple_PointCloud2\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera_front_001\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_71\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera_front_002\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_72\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera003\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_73\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera004\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_74\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera005\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_75\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera006\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_76\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera007\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_77\"},{\"sensor_type\":\"camera\",\"sensor_type_id\":\"camera008\",\"topic_type\":\"roscpp_tutorials/VideoEncodedData\",\"topic_path\":\"/camera/encode_h265_78\"}]}"}
    , "parentContext": [
        {"dataPartitionId": "instanceCode_test_2022083101_PARTITION_1",
         "userContext": {"A": "A1"}}]}


def execute():
    # 拆分节
    partition_data["instanceCode"] = str(time.time_ns())
    partition_data["workspace"] = workspace
    partition_data["parameters"]["ossPrefix"] = oss_prefix
    http_post(CONNECT_ADDRESS + INSTANCE_RUN, partition_data)

    # 状态校验，拆分完成后继续
    check_status(partition_data["instanceCode"])

    # 获取拆分结果
    parallel_data["instanceCode"] = str(time.time_ns())
    parallel_data["workspace"] = workspace
    parallel_data["transformParams"] = transform_params
    data_partition_ids = get_data_partition_ids(partition_data["instanceCode"])
    for partition_id in data_partition_ids:
        parallel_data["dataPartitionId"] = partition_id
        http_post(CONNECT_ADDRESS + INSTANCE_RUN, parallel_data)


# 校验状态
# running 持续校验
# failed/killed 抛异常
# succeed 返回成功
def check_status(instance_code):
    instance = {"instanceCode": instance_code}
    status = http_get(CONNECT_ADDRESS + GET_STATUS, instance)["status"]
    if status == "running":
        Timer(1, check_status, ()).start()

    if status == "failed" or status == "killed":
        raise Exception("Status fail, status is:" + status)

    return


def get_data_partition_ids(instance_code):
    instance = {"instanceCode": instance_code}
    return http_post(CONNECT_ADDRESS + GET_STATUS, instance)[DATA_PARTITION_ID_LIST]
