import requests
import json
import os
from ali_autodrive.parallel_compute.executor_agent.LogUtil import *

# http连接信息
# 本地http服务,约定服务端口号9001
CONNECT_ADDRESS = "http://127.0.0.1:9001"
# GET请求超时时间，5秒
CONNECT_TIME_OUT_GET = 5
# POST请求超时时间，考虑保存及加载数据耗时较久，且走本地网络，超时设置为300秒
CONNECT_TIME_OUT_POST = 300

# http请求返回成功
RESPONSE_SUCCESS = "success"
# http请求返回数据
RESPONSE_DATA = "data"
# 方法执行结果，正确
RESULT_TRUE = "true"
# 环境变量
AUTO_DRIVE_ENV = "autoDriveEnv"
AUTO_DRIVE_CONNECT_ADDRESS_DEV = "http://11.164.61.242:7001"
AUTO_DRIVE_CONNECT_ADDRESS_PRE = "http://172.16.1.10:7001"
AUTO_DRIVE_CONNECT_ADDRESS_PRODUCT = "http://172.16.1.9:7001"


def create_data_partition(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/create', data)


def save_data_partition(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/save', data)


def submit_data_partition(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/submit', data)


def load_unstructured_data_partition(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/unstructured/load', data)


def get_data_partition(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/get', data)


def get_partition_local_desc_file(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/localDescFile/get', data)


def scan_file_tree(data):
    return http_post(CONNECT_ADDRESS + '/autoDrive/parallelCompute/dataPartition/fileTree/scan', data)


# 数据集请求
def create_data_set(data):
    return http_post(__get_host() + '/autoDrive/taskflow/parallelCompute/createDataSet', data)


def get_data_set(data):
    return http_get(__get_host() + '/autoDriver/dataset/detail', data)


def http_post(url, data):
    try:
        headers = {
            'Content-Type': 'application/json',
            'citybrain-user-profile': '{\"userId\":\"parallelCompute\",\"workspaceCode\":\"parallelCompute\",\"tenantCode\":\"parallelCompute\"}'
        }
        default_logger.info("http_post url:" + url + " data:" + data)
        rsp = requests.post(url, data=data, headers=headers, timeout=CONNECT_TIME_OUT_POST)
        default_logger.info("http_post rsp:" + json.dumps(rsp.json()))
        res_json = json.loads(json.dumps(rsp.json()))

        if "status" in res_json and res_json["status"] != 200:
            raise RuntimeError("Http post request fail.")

        if "code" in res_json and res_json["code"] != "SUCCESS":
            raise RuntimeError("Http post request process fail.")

        return res_json[RESPONSE_DATA]
    except:
        raise RuntimeError("Http post request exception.")


def http_get(url, data):
    try:
        headers = {
            'Content-Type': 'application/json',
            'citybrain-user-profile': '{\"userId\":\"parallelCompute\",\"workspaceCode\":\"parallelCompute\",\"tenantCode\":\"parallelCompute\"}'
        }

        default_logger.info("http_get url:" + url + " data:" + json.dumps(data))
        rsp = requests.get(url, params=data, headers=headers, timeout=CONNECT_TIME_OUT_GET)
        default_logger.info("http_get rsp:" + json.dumps(rsp.json()))
        res_json = json.loads(json.dumps(rsp.json()))

        if "status" in res_json and res_json["status_code"] != 200:
            raise RuntimeError("Http get request fail.")

        if "code" in res_json and res_json["code"] != "SUCCESS":
            raise RuntimeError("Http get request process fail.")

        return res_json[RESPONSE_DATA]
    except:
        raise RuntimeError("Http get request exception.")


def __get_host():
    env = os.getenv(AUTO_DRIVE_ENV)
    if env is None:
        raise Exception("Get host fail. env:" + json.dumps(env))

    if env == 'DEV':
        return AUTO_DRIVE_CONNECT_ADDRESS_DEV

    if env == 'PRE':
        return AUTO_DRIVE_CONNECT_ADDRESS_PRE

    if env == 'PRODUCT':
        return AUTO_DRIVE_CONNECT_ADDRESS_PRODUCT

    raise Exception("Get host fail. env:" + json.dumps(env))
